﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContainerShip
{
    internal class DrawingObjectShip : IDrawingObject
    {
        private DrawingShip _ship = null;
        public DrawingObjectShip(DrawingShip ship)
        {
            _ship = ship;
        }
        public DrawingShip GetShip => _ship;
        public float Step => _ship?.Ship?.Step ?? 0;

        public (float Left, float Top, float Right, float Bottom) GetCurrentPosition()
        {
            return _ship?.GetCurrentPosition() ?? default;
        }

        public void MoveObject(Direction direction)
        {
            _ship?.MoveTransport(direction);
        }

        public void SetObject(int x, int y, int width, int height)
        {
            _ship.SetPosition(x, y, width, height);
        }
        void IDrawingObject.DrawingObject(Graphics g)
        {
            if (_ship == null) return;
            if (_ship is DrawingContainerShip ship)
            {
                ship.DrawTransport(g);
            }
            else
            {
                _ship.DrawTransport(g);
            }
        }
        public string GetInfo() => _ship?.GetDataForSave();
        public static IDrawingObject Create(string data) => new DrawingObjectShip(data.CreateDrawingShip());

        public bool Equals(IDrawingObject? other)
        {   
                if (other == null)
                {
                    return false;
                }
                var otherShip = other as DrawingObjectShip;
                if (otherShip == null)
                {
                    return false;
                }
                var ship = _ship.Ship;
                var otherShipShip = otherShip._ship.Ship;
                if (ship.Speed != otherShipShip.Speed)
                {
                    return false;
                }
                if (ship.Weight != otherShipShip.Weight)
                {
                    return false;
                }
                if (ship.BodyColor != otherShipShip.BodyColor)
                {
                    return false;
                }
                if (ship is EntityContainerShip containerShip && otherShipShip is EntityContainerShip otherContainerShip)
                {
                    if (containerShip.DopColor != otherContainerShip.DopColor)
                    {
                        return false;
                    }
                    if (containerShip.Containers != otherContainerShip.Containers)
                    {
                        return false;
                    }
                    if (containerShip.Crane != otherContainerShip.Crane)
                    {
                        return false;
                    }
                }
                else if (ship is EntityContainerShip || otherShipShip is EntityContainerShip) return false;
            return true;
        }
    }
}
