﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ContainerShip
{
    internal class SetShipsGeneric<T>
    where T : class, IEquatable<T>
    {
        /// <summary>
		///  Список объектов, которые храним
		/// </summary>
		private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        private readonly int _maxCount;
        public SetShipsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="car">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T ship)
        {
            return Insert(ship, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="ship">Добавляемый корабль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T ship, int position)
        {
            foreach (var checkShip in _places)
            {
                if (checkShip.Equals(ship)) {
                    throw new ArgumentException();
                }
                
            }
            if (position < 0 || position >= _maxCount)
            {
                throw new ShipNotFoundException(position);
            }
            if ( _places.Count >= _maxCount)
            {
                throw new StorageOverflowException(_maxCount);
            }
            _places.Add(ship);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {   
            if (position < 0 || position >= _maxCount)
            {
                throw new ShipNotFoundException(position);
            }
            T ship = _places[position];
            _places.RemoveAt(position);
            return ship;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                // TODO проверка позиции
                if (position < 0 || position > _maxCount) return null;
                return _places[position];
            }
            set
            {
                if (position >= 0 || position < _maxCount)
                {
                    Insert(value, position);
                }
                // TODO проверка позиции
                // TODO вставка в список по позиции
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetShips()
        {
            foreach (var ship in _places)
            {
                if (ship != null)
                {
                    yield return ship;
                }
                else
                {
                    yield break;
                }
            }
        }
        public void SortSet(IComparer<T> comparer)
		{
			if (comparer == null)
			{
				return;
			}
			_places.Sort(comparer);
		}
}
}
